import java.io.*;
import java.util.Date;
import java.text.DateFormat;
import java.util.ArrayList;

    /* Maintain a File object relating to a particular directory (folder).
     * Construction will fail if either the given
     * directory name does not exist or the name does not relate to a
     * directory. Methods are provided to print a simple list of the
     * directory's contents, or a more detailed list including the type
     * and permissions for each file, the date on which it was last
     * modified and its name.
     * printNestedContents() prints the contents of this directory and
     * those of all subdirectories.
     */

class RecursiveLister {
    // Construct from the name of a directory.
    public RecursiveLister(String dirname)
                        throws FileNotFoundException, Exception {
        this(new File(dirname));
    }

    // Construct from a File object that should refer to a directory.
    public RecursiveLister(File dir)
                        throws FileNotFoundException, Exception {
        setDirectory(dir);
    }

        /* Print the contents of this directory and all nested
         * directories. This is not handled by recursion of this method
         * but by recursively creating further RecursiveLister
         * objects for each sub directory.
         */
    public void printNestedContents(){
        // Print the contents of this directory.
        printContents();
        // Find each directory in the current directory.
        File[] directories = getSubDirectories();
        final int numDirectories = directories.length;
        for(int i = 0; i < numDirectories; i++){
            try{
                RecursiveLister subdir = new RecursiveLister(directories[i]);
                subdir.printNestedContents();
            }
            catch(Exception e){
                System.err.println(e);
            }
        }
    }

        // Return an array of File objects representing the
        // sub-directories within this directory.
        // Return an array of all the subdirectories within
        // this directory.
    public File[] getSubDirectories(){
        ArrayList directories = new ArrayList();
        File details = getDirectory();
        final String path = getPath();
        String[] list = details.list();
        final int numFiles = list.length;
        for(int i = 0; i < numFiles; i++){
            File f = new File(path+File.separator+list[i]);
            if(f.isDirectory()){
                directories.add(f);
            }
        }
        // Return the directories collection as an array.
        File[] fileArray = new File[directories.size()];
        directories.toArray(fileArray);
        return fileArray;
    }

        // Print the contents of this directory. Only print
        // the names, one per line, without further details.
    public void printContents(){
        // Include the directory's full path as a heading.
        System.out.println(getPath());
        File details = getDirectory();

        String[] list = details.list();
        final int numFiles = list.length;
        for(int i = 0; i < numFiles; i++){
            System.out.println(list[i]);
        }
    }

        // Print details of the contents of this directory.
        // Include file type, read/write permission and date
        // of last modification.
        // This appears in the form
        //        frw Dec 25, 1999 filename

    public void printFullDetails(){
        // Format the file modification time.
        DateFormat formatter = DateFormat.getDateInstance();
        // Obtain the path of this directory.
        String path = getPath();
        File details = getDirectory();
        // Obtain the list of file objects for the contents.
        File[] list = details.listFiles();
        final int numFiles = list.length;

        System.out.println(path);
        for(int i = 0; i < numFiles; i++){
            final File f = list[i];
            // Don't include hidden files
            if(!f.isHidden()){
                String info = "";
                info += f.isDirectory() ? "d" : f.isFile() ? "f" : "-";
                info += f.canRead() ? "r" : "-";
                info += f.canWrite() ? "w" : "-";

                // Include the date on which the file was last modified.
                Date modified = new Date(f.lastModified());
                info += " "+formatter.format(modified);
                info += " "+f.getName();
                System.out.println(info);
            }
        }
    }

        // Try to return the canonical path, but use the absolute
        // path if that fails.
    public String getPath(){
        File details = getDirectory();
        String path = "";
        try{
            path = details.getCanonicalPath();
        }
        catch(IOException e){
            // Cannot obtain the canonical path, for some reason.
            // Use the absolute path.
            path = details.getAbsolutePath();
        }
        return path;
    }

        // Check that the file exists and is a directory.
    public void checkForDirectory(File f)
                        throws FileNotFoundException, Exception {
        if(!f.exists()){
            throw new FileNotFoundException(f.getName());
        }
        else if(!f.isDirectory()){
            throw new Exception(f.getName()+" is not a directory.");
        }
        // else - ok.
    }

    public File getDirectory(){
        return directory;
    }

        // Set the Directory to be used. This must be a directory.
    public void setDirectory(File d) throws FileNotFoundException, Exception {
        checkForDirectory(d);
        directory = d;
    }

    // Assert: directory.exists() && directory.isDirectory().
    private File directory;
}
